<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/src/Utils/Translator.php'; //autoloader issues during upgrade from version < 2.0
require_once __DIR__ . '/src/PluginConfig.php'; //autoloader issues during upgrade from version < 2.0
require_once __DIR__ . '/vendor/autoload.php';

class Kevin extends PaymentModule
{
    const MODULE_NAME = 'kevin';
    const MODULE_FILE = __FILE__;
    const HOOKS = [
        'header',
        'backOfficeHeader',
        'displayAdminOrderContentShip',
        'displayAdminOrderTabShip',
        'displayAdminOrderTabContent',
        'displayAdminOrderTabLink',
        'displayOrderConfirmation',
        'orderConfirmation',
        'paymentOptions',
        'payment',
        'actionOrderSlipAdd',
        'actionProductCancel',
    ];

    /**
     * Due to vendor autoloader problems during upgrading from 1.10 < version to 2.0 I cant initialize this in constructor
     * Moved the initialization to getter (sic!)
     *
     * @var \KevinPlugin\GatewaySettings|null
     */
    private $gatewaySettings;

    /**
     * Due to vendor autoloader problems during upgrading from 1.10 < version to 2.0 I cant initialize this in constructor
     * Moved the initialization to getter (sic!)
     *
     * @var \KevinPlugin\Gateway|null
     */
    private $gateway;

    public function __construct()
    {
        $this->name = 'kevin';
        $this->tab = 'payments_gateways';
        $this->version = '2.2.3';
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => '8.99'];
        $this->author = 'kevin.';
        $this->controllers = ['redirect', 'confirm', 'webhook'];
        $this->bootstrap = true;
        $this->module_key = '46c14947cfed9adf2e72c8f703b54429';

        parent::__construct();

        $this->displayName = 'kevin.';
        $this->description = $this->l('Let your customers make fast, simple and secure payments directly from their bank accounts across Europe.');
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
    }

    /**
     * @return bool
     */
    public function install()
    {
        \KevinPlugin\Setup\DbSetup::install();
        \KevinPlugin\Setup\KevinOrderStatusSetup::install();

        if (!parent::install()) {
            return false;
        }

        foreach (self::HOOKS as $hook) {
            if ($this->isRegisteredInHook($hook)) {
                continue;
            }
            if (!$this->registerHook($hook)) {
                return false;
            }
        }

        (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_INSTALLED);

        return true;
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        if (!parent::uninstall()) {
            return false;
        }

        foreach (self::HOOKS as $hook) {
            $this->unregisterHook($hook);
        }

        (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_UNINSTALLED);

        return true;
    }

    public function reset()
    {
        if (!$this->uninstall()) {
            return false;
        }
        if (!$this->install()) {
            return false;
        }

        return true;
    }

    public function enable($force_all = false)
    {
        $enabled = parent::enable($force_all);
        (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_ACTIVATED);

        return $enabled;
    }

    public function disable($force_all = false)
    {
        $disabled = parent::disable($force_all);
        (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_DEACTIVATED);

        return $disabled;
    }

    protected static function loadUpgradeVersionList($module_name, $module_version, $registered_version)
    {
        require_once __DIR__ . '/src/GatewaySettings.php';
        require_once __DIR__ . '/src/PluginConfig.php';
        require_once __DIR__ . '/src/Client/MonitoringClient.php';

        (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_UPDATED);

        return parent::loadUpgradeVersionList($module_name, $module_version, $registered_version);
    }

    /**
     * @return bool
     */
    public function isAvailable()
    {
        return $this->active && $this->getGateway() && $this->getGateway()->isEnabled();
    }

    /**
     * @return \KevinPlugin\Gateway|null
     */
    public function getGateway($init = false)
    {
        if (!$this->gateway || $init === true) {
            try {
                $this->gateway = \KevinPlugin\Gateway::init($this->getGatewaySettings($init));
            } catch (\Exception $e) {
                $this->gateway = null;
            }
        }
        return $this->gateway;
    }

    /**
     * @return \KevinPlugin\GatewaySettings
     */
    public function getGatewaySettings($init = false)
    {
        if (!$this->gatewaySettings || $init === true) {
            $this->gatewaySettings = new \KevinPlugin\GatewaySettings();
        }
        return $this->gatewaySettings;
    }

    /**
     * Load the admin configuration form.
     *
     * @return string
     *
     * @throws SmartyException
     */
    public function getContent()
    {
        $settingsForm = new \KevinPlugin\Form\Settings\SettingsForm($this);

        if ($settingsForm->isSubmitted()) {
            if ($settingsForm->validate()) {
                $settingsForm->save();
                if (!$this->getGateway(true)->getKevinProjectSettings()) {
                    \Configuration::updateValue(\KevinPlugin\GatewaySettings::KEVIN_ENABLED, 0);
                }
                $settingsForm->populate();
                (new \KevinPlugin\Client\MonitoringClient())->sendEvent(\KevinPlugin\Client\MonitoringClient::EVENT_SETTINGS_CHANGED);
            }
        }

        return $settingsForm->html($this->getGateway());
    }

    /**
     * Hook files for storefront header.
     */
    public function hookHeader()
    {
        if ($this->context->controller->php_self !== 'order') {
            return;
        }

        $this->context->controller->addCSS($this->_path . 'views/css/checkout.css');
        $this->context->controller->addCSS($this->_path . 'views/css/kevin-styles.css');

        $this->context->controller->addJs($this->_path . 'views/js/checkout.js');

        if (\KevinPlugin\Utils\PsVersionHelper::isPre17Version()) {
            return;
        }

        if ($this->getGatewaySettings()->isCheckoutViewList()) {
            $this->context->controller->addCSS($this->_path . 'views/css/kevin-bank-list.css');
        }

        if ($this->getGatewaySettings()->isCheckoutViewTiles()) {
            $this->context->controller->addCSS($this->_path . 'views/css/kevin-bank-grid.css');
        }
    }

    /**
     * Hook files for admin header.
     */
    public function hookBackOfficeHeader()
    {
        $orderId = Tools::getValue('id_order');
        if (!$orderId || !$this->isAvailable() || !$this->getGateway()->isRefundAllowed()) {
            return;
        }
        $order = new Order($orderId);
        if ($order->module !== $this->name) {
            return;
        }

        $kevinRefundRepository = new \KevinPlugin\Repository\KevinRefundRepository();

        $availableForRefund = (float) $order->total_paid - $kevinRefundRepository->getSumRefundsForOrder($orderId);

        if ($availableForRefund <= 0 || $order->getCurrentState() == Configuration::get('PS_OS_REFUND')) {
            return;
        }

        $this->context->controller->addJquery();
        $this->context->controller->addJs($this->_path . 'views/js/admin_refunds.js');

        Media::addJsDefL('kevin_order_id', $orderId);
        Media::addJsDefL('kevin_refund_form_token', uniqid());
        Media::addJsDefL('kevin_order_total', number_format($order->total_paid, 2));
        Media::addJsDefL('kevin_max_refund_amount', number_format($availableForRefund, 2));
        Media::addJsDefL('kevin_order_currency', (new Currency($order->id_currency))->getSign());
        Media::addJsDefL('kevin_text', \KevinPlugin\Utils\Translator::translate('Process the refund using kevin. payment infrastructure'));
        Media::addJsDefL('kevin_refund_exceeds_text', \KevinPlugin\Utils\Translator::translate('Unable to process refund via kevin.: amount exceeds original payment.'));
        Media::addJsDefL('kevin_refund_zero_text', \KevinPlugin\Utils\Translator::translate('Unable to process refund via kevin.: amount must be greater than zero.'));
        Media::addJsDefL('kevin_refund_amount_text', \KevinPlugin\Utils\Translator::translate('The exact amount that will be returned via kevin.').':');
    }

    /**
     * CHECKOUT PAYMENT VIEW FOR PRESTASHOP 1.6
     *
     * @param $params
     * @return false|string
     */
    public function hookPayment($params)
    {
        if (!$this->isAvailable()) {
            return false;
        }

        $view = new \KevinPlugin\View\Payment16\PaymentView($this->getGateway(), $this->getGatewaySettings());

        return $view->html($this);
    }

    /**
     * CHECKOUT PAYMENT VIEW FOR PRESTASHOP 1.7+
     *
     * @param $params
     *
     * @return array
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws SmartyException
     */
    public function hookPaymentOptions($params)
    {
        if (empty($params['cart']) || !$this->isAvailable()) {
            return [];
        }

        /** @var Cart $cart */
        $cart = $params['cart'];

        $options = [];

        if ($this->getGatewaySettings()->isCheckoutViewFrame()) {
            $options[] = \KevinPlugin\View\Payment17\RedirectPaymentOptionBuilder::build($this->getGateway(), $this->getGatewaySettings());
        } elseif ($this->getGateway()->isBankPaymentEnabled()) {
            $options[] = \KevinPlugin\View\Payment17\BankPaymentOptionBuilder::build($this->getGateway(), $this->getGatewaySettings(), $cart);
        }
        if ($this->getGateway()->isCardPaymentEnabled()) {
            $options[] = \KevinPlugin\View\Payment17\CardPaymentOptionBuilder::build($this->getGateway(), $this->getGatewaySettings());
        }

        return $options;
    }

    /**
     * Triggerred after Refund process is done by Prestashop (PS 1.6+)
     *
     * @param $params
     *
     * @return void
     */
    public function hookActionOrderSlipAdd($params)
    {
        if (!Tools::isSubmit('partialRefund') || !isset($params['order']) || !Tools::getValue('refundwithkevin')) {
            return;
        }

        $this->processRefund($params['order']);
    }

    /**
     * Triggerred after Refund process is done by Prestashop (late 1.7 versions. Probably 1.7.6+)
     *
     * @param $params
     *
     * @return void
     */
    public function hookActionProductCancel($params)
    {
        if (!Tools::isSubmit('cancel_product') || !isset($params['order']) || !Tools::getValue('refundwithkevin')) {
            return;
        }

        $this->processRefund($params['order']);
    }

    public function hookOrderConfirmation($params)
    {
        if (isset($params['order'])) {
            $order = $params['order'];
        } elseif (isset($params['objOrder'])) {
            $order = $params['objOrder'];
        } else {
            $order = null;
        }
        /** @var Order $order */
        if ($order->module !== $this->name) {
            return;
        }

        $view = new \KevinPlugin\View\OrderConfirmationView(Tools::getValue('statusGroup'));
        return $view->html($this);
    }

    /**
     * (removed in 1.7.7 in favor of) → @see hookDisplayAdminOrderTabLink
     */
    public function hookDisplayAdminOrderTabShip()
    {
        $orderId = Tools::getValue('id_order');
        $order = new Order($orderId);
        if ($order->module !== $this->name) {
            return;
        }

        return $this->display(__FILE__, 'views/templates/hook/admin_order_tab_ship.tpl');
    }

    public function hookDisplayAdminOrderTabLink()
    {
        return $this->hookDisplayAdminOrderTabShip();
    }

    /**
     * (removed in 1.7.7 in favor of) → @see hookDisplayAdminOrderTabContent
     */
    public function hookDisplayAdminOrderContentShip()
    {
        $orderId = Tools::getValue('id_order');
        $order = new Order($orderId);
        if ($order->module !== $this->name) {
            return;
        }
        $kevinPaymentRepository = new \KevinPlugin\Repository\KevinPaymentRepository();
        $kevinPayment = $kevinPaymentRepository->getByOrderId($orderId);
        if (!$kevinPayment) {
            return;
        }
        $kevinRefundRepository = new \KevinPlugin\Repository\KevinRefundRepository();
        $kevinRefunds = $kevinRefundRepository->getAllByPaymentId($kevinPayment['payment_id']);
        $view = new \KevinPlugin\View\AdminOrderKevinTabView($order, $kevinPayment, $kevinRefunds);

        return $view->html($this);
    }

    public function hookDisplayAdminOrderTabContent()
    {
        return $this->hookDisplayAdminOrderContentShip();
    }

    /**
     * @param Order $order
     *
     * @return bool
     */
    private function processRefund(Order $order)
    {
        if (!\KevinPlugin\Utils\UserHelper::isEmployee()) {
            return false;
        }

        $refundFormToken = Tools::getValue('refundwithkevin-token');
        if (!$refundFormToken) {
            return false;
        }

        $refundAmount = (float) Tools::getValue('refundwithkevin-amount');
        if ($refundAmount <= 0) {
            return false;
        }

        $handler = new \KevinPlugin\Handler\RefundHandler($this->getGateway(), new \KevinPlugin\Repository\KevinPaymentRepository(), new \KevinPlugin\Repository\KevinRefundRepository());

        return $handler->handle($refundAmount, $order, $refundFormToken);
    }
}
