<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace KevinPlugin\Client;

use Configuration;
use Context;
use KevinPlugin\GatewaySettings;
use KevinPlugin\PluginConfig;
use Tools;

class MonitoringClient
{
    const EVENT_INSTALLED = 'installed';
    const EVENT_UNINSTALLED = 'uninstalled';
    const EVENT_UPDATED = 'updated';
    const EVENT_ACTIVATED = 'activated';
    const EVENT_DEACTIVATED = 'deactivated';
    const EVENT_SETTINGS_CHANGED = 'settings_changed';
    const KEVIN_MONITORING_INSTALLATION_ID = 'KEVIN_MONITORING_INSTALLATION_ID';

    /**
     * @var string
     */
    private $url;
    /**
     * @var string
     */
    private $authKey;
    /**
     * @var string
     */
    private $installationId;
    /**
     * @var string|null
     */
    private $clientId;
    /**
     * @var bool
     */
    private $enabled;

    public function __construct()
    {
        if ($this->url = PluginConfig::KEVIN_MONITORING_SERVICE_URL()) {
            $this->url = rtrim($this->url, '/') . '/';
        }
        $this->authKey = PluginConfig::KEVIN_MONITORING_SERVICE_KEY();
        $this->enabled = $this->url && $this->authKey;
        $this->installationId = Configuration::get(self::KEVIN_MONITORING_INSTALLATION_ID) ?: null;
        if (!$this->installationId) {
            $installationId = uniqid('', true);
            Configuration::updateValue(self::KEVIN_MONITORING_INSTALLATION_ID, $installationId);
            $this->installationId = $installationId;
        }
        $this->clientId = Configuration::get(GatewaySettings::KEVIN_CLIENT_ID) ?: null;
    }


    public function sendEvent($type)
    {
        if (!$this->enabled) {
            return;
        }
        $data = [
            'type' => $type,
            'shop' => [
                'url' => Context::getContext()->link->getBaseLink(),
                'systemType' => 'php',
                'systemVersion' => phpversion(),
                'platformName' => 'prestashop',
                'platformVersion' => _PS_VERSION_
            ],
            'integration' => [
                'version' => PluginConfig::PLUGIN_VERSION(),
                'installationId' => $this->installationId,
                'clientId' => $this->clientId,
            ],
            'occurredAt' => microtime(true) * 1000
        ];

        if ($this->clientId) {
            $data['integration']['settings'] = $this->getPluginSettings();
        }

        $this->sendPostRequest('events', $data);
    }

    private function getPluginSettings()
    {
        return [
            'enabled' => (bool) Configuration::get(GatewaySettings::KEVIN_ENABLED),
            'checkoutView' => Configuration::get(GatewaySettings::KEVIN_CHECKOUT_VIEW) ?: null,
            'countryDropdown' => (bool) Configuration::get(GatewaySettings::KEVIN_COUNTRY_DROPDOWN),
            'bankSearch' => (bool) Configuration::get(GatewaySettings::KEVIN_BANK_SEARCH),
        ];
    }

    private function sendPostRequest($route, array $data)
    {
        $options = [
            'http' => [
                'header'  => "Content-Type: application/json\r\n" .
                    "X-Kevin-System-Secret: {$this->authKey}\r\n",
                'method'  => 'POST',
                'timeout' => 5,
                'content' => json_encode($data)
            ]
        ];

        $context = stream_context_create($options);
        $result = @Tools::file_get_contents($this->url.$route, false, $context);

        if ($result === false) {
            return [
                'isSuccess' => false,
                'data'   => null
            ];
        }

        return [
            'isSuccess' => true,
            'data'   => json_decode($result)
        ];
    }
}