<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace KevinPlugin\Handler;

if (!defined('_PS_VERSION_')) {
    exit;
}

use Cart;
use Configuration;
use Currency;
use Customer;
use Kevin;
use KevinPlugin\Exception\OrderCreationException;
use KevinPlugin\Repository\KevinPaymentRepository;
use KevinPlugin\Utils\OrderHelper;
use KevinPluginFramework\Application\KevinConstants;
use Order;
use PrestaShopException;

class OrderCreationHandler
{
    /**
     * @var \Kevin
     */
    private $module;
    /**
     * @var KevinPaymentRepository
     */
    private $kevinPaymentRepository;

    public function __construct(Kevin $module, KevinPaymentRepository $kevinPaymentRepository)
    {
        $this->module = $module;
        $this->kevinPaymentRepository = $kevinPaymentRepository;
    }

    /**
     * @param Cart $cart
     * @param $kevinPaymentId
     * @return Order
     * @throws OrderCreationException
     * @throws PrestaShopException
     * @throws \KevinPluginFramework\Application\Exception\KevinRequestException
     * @throws \PrestaShopDatabaseException
     */
    public function handle(Cart $cart, $kevinPaymentId)
    {
        $kevinPaymentResponse = $this->module->getGateway()->getPayment($kevinPaymentId);
        $kevinPaymentDbRecord = $this->kevinPaymentRepository->getByPaymentId($kevinPaymentId);

        if ($kevinPaymentResponse['statusGroup'] === KevinConstants::STATUS_GROUP_FAILED ||
            $kevinPaymentResponse['statusGroup'] === KevinConstants::STATUS_GROUP_STARTED) {
            throw new OrderCreationException('Order couldnt be created.');
        }

        if ($kevinPaymentDbRecord['id_order'] || $cart->orderExists() || OrderHelper::getOrderIdByCartId($cart->id)) {
            throw new OrderCreationException('Order has already been placed using this cart.');
        }

        $order = $this->createOrderFromKevinPaymentApiResponse($cart, $kevinPaymentResponse);

        $this->kevinPaymentRepository->updateOrderId($kevinPaymentId, $order->id);
        $this->kevinPaymentRepository->updatePaymentStatus($kevinPaymentId, $kevinPaymentResponse['statusGroup']);

        return $order;
    }


    /**
     * @param Cart $cart
     * @param array $kevinPayment
     * @return Order
     * @throws PrestaShopException
     * @throws \PrestaShopDatabaseException]
     */
    private function createOrderFromKevinPaymentApiResponse(Cart $cart, array $kevinPayment)
    {
        $cartId = (int) $cart->id;
        $cartCustomer = new Customer($cart->id_customer);

        $this->module->validateOrder(
            $cartId,
            $this->mapOrderStatus($kevinPayment['statusGroup']),
            (float) $kevinPayment['amount'],
            $this->getPaymentMethodName($kevinPayment, $cartId),
            null,
            ['transaction_id' => $kevinPayment['id']],
            Currency::getIdByIsoCode($kevinPayment['currencyCode']),
            false,
            $cartCustomer->secure_key
        );

        return new Order(OrderHelper::getOrderIdByCartId($cartId));
    }

    private function getPaymentMethodName(array $kevinPayment, $cartId)
    {
        $method = $this->module->displayName;
        $bankId = isset($kevinPayment['bankPaymentMethod']['bankId']) ? $kevinPayment['bankPaymentMethod']['bankId'] : null;
        if ($bankId) {
            $bank = $this->module->getGateway()->getKevinBank($bankId);
            if (!empty($bank['officialName'])) {
                $method = $bank['officialName'];
            }
        } elseif (isset($kevinPayment['cardPaymentMethod'])) {
            $method = 'Credit/Debit card';
        }


        return "$method (CART ID: $cartId)";
    }

    /**
     * @param $kevinStatus
     *
     * @return int
     */
    private function mapOrderStatus($kevinStatus)
    {
        switch ($kevinStatus) {
            case KevinConstants::STATUS_GROUP_COMPLETED:
                return (int) Configuration::get('PS_OS_PAYMENT');
            case KevinConstants::STATUS_GROUP_FAILED:
                return (int) Configuration::get('PS_OS_ERROR');
            default:
                return $this->module->getGatewaySettings()->getOrderStatusKevinPendingId();
        }
    }
}
