<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License version 3.0
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License version 3.0
 */

namespace KevinPlugin\View\Payment16;

use Address;
use Context;
use Country;
use KevinPlugin\Gateway;
use KevinPlugin\GatewaySettings;
use KevinPlugin\Utils\Translator;

class PaymentView
{
    /**
     * @var Gateway
     */
    private $gateway;
    /**
     * @var GatewaySettings
     */
    private $gatewaySettings;
    /**
     * @var Context
     */
    private $context;

    public function __construct(Gateway $gateway, GatewaySettings $gatewaySettings)
    {
        $this->gateway = $gateway;
        $this->gatewaySettings = $gatewaySettings;
        $this->context = Context::getContext();
    }


    public function html(\Kevin $module)
    {
        $alerts = [];
        if ($this->gateway->isSandbox()) {
            $alerts[] = Translator::translate('kevin. payment gateway is in test mode for testing purposes. Real payments are not available.');
        }

        if ($this->gateway->isCardPaymentEnabled()) {
            $this->context->smarty->assign([
                'card_payment_img_uri' => 'https://cdn.kevin.eu/banks/images/VISA_MC.png',
                'card_payment_text_title' => Translator::translate('Credit/Debit card'),
                'card_payment_redirect_url' => $this->context->link->getModuleLink($module->name, 'redirect', ['id' => 'card'], true),
            ]);
        }

        if ($this->gatewaySettings->isCheckoutViewFrame()) {
            $this->context->smarty->assign([
                'redirect_to_frame_title' => Translator::translate('Bank payments'),
                'redirect_to_frame_redirect_url' => $this->context->link->getModuleLink($module->name, 'redirect', ['id' => ''], true)
            ]);
        } elseif ($this->gateway->isBankPaymentEnabled()) {

            $address = new Address($this->context->cart->id_address_invoice);
            $country = new Country($address->id_country, $this->context->cart->id_lang);

            $availableCountries = $this->formatCountryCodesList($this->gateway->getKevinCountries(), $this->context->cart->id_lang);

            $isCountryAvailableInKevin = isset($availableCountries[$country->iso_code]);

            if (!$this->gatewaySettings->isCountryDropdown() && !$isCountryAvailableInKevin) {
                $alerts[] = sprintf(
                    Translator::translate('No bank is available in your selected country: %s'), $country->name
                );
            }

            $banks = [];
            foreach ($this->gateway->getKevinBanks() as $kevinBank) {
                $kevinBank['redirect_url'] = $this->context->link->getModuleLink($module->name, 'redirect', ['id' => $kevinBank['id']], true);
                $banks[] = $kevinBank;
            }

            $this->context->smarty->assign([
                'is_country_dropdown' => $this->gatewaySettings->isCountryDropdown(),
                'is_bank_search' => $this->gatewaySettings->isBankSearch(),
                'country_codes' => $availableCountries,
                'selected_country_code' => $isCountryAvailableInKevin ? $country->iso_code : null,
                'banks' => $banks,
                'text_bank_search_placeholder' => Translator::translate('Bank name'),
                'text_legal_info' => Translator::translate('By using the payment initiation service provided by Kevin EU, UAB, you confirm that you have read and agree with:'),
                'text_legal_privacy_policy' => Translator::translate('Privacy policy'),
                'text_legal_tos' => Translator::translate('Terms and conditions'),
                'text_select_country' => Translator::translate('Select country'),
            ]);
        }

        $this->context->smarty->assign([
            'alerts' => $alerts,
            'is_card_payment' => $this->gateway->isCardPaymentEnabled(),
            'is_redirect_to_frame_payment' => $this->gatewaySettings->isCheckoutViewFrame(),
            'is_bank_payment' => $this->gateway->isBankPaymentEnabled(),
        ]);

        return $module->display(\Kevin::MODULE_FILE, 'views/templates/hook/ps16/payment.tpl');
    }

    /**
     * @param array $kevinCountryCodes
     * @param int $langId
     *
     * @return array
     */
    private function formatCountryCodesList($kevinCountryCodes, $langId)
    {
        $countryCodes = [];
        foreach ($kevinCountryCodes as $kevinCountryCode) {
            $countryCodes[$kevinCountryCode] = Country::getNameById($langId, Country::getByIso($kevinCountryCode));
        }
        $countryCodes = array_filter(array_unique($countryCodes));
        asort($countryCodes);

        return $countryCodes;
    }
}